//---------------------------------------------------------------------------------
// Copyright (c) 2011, Microsoft Corporation
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//---------------------------------------------------------------------------------

namespace Microsoft.Samples.AsyncMessaging
{
    using System;
    using System.Threading;
    using Microsoft.ServiceBus;
    using Microsoft.ServiceBus.Messaging;

    public class AsyncReceiver
    {
        const string QueueName = "IssueTrackingQueue";
        static string ServiceNamespace;
        static string IssuerName;
        static string IssuerKey;

        static void Main(string[] args)
        {
            GetUserCredentials();
            TokenProvider credentials = TokenProvider.CreateSharedSecretTokenProvider(AsyncReceiver.IssuerName, AsyncReceiver.IssuerKey);
            Uri serviceUri = ServiceBusEnvironment.CreateServiceUri("sb", AsyncReceiver.ServiceNamespace, string.Empty);

            MessagingFactory factory = null;

            try
            {
                //*****************************************************************************************************
                //                                   Management Operations
                //*****************************************************************************************************           
                NamespaceManager namespaceClient = new NamespaceManager(serviceUri, credentials);
                if (namespaceClient == null)
                {
                    Console.WriteLine("\nUnexpected Error: NamespaceManager is NULL");
                    return;
                }

                //Retreive details of the queue 
                QueueDescription queueDescription = namespaceClient.GetQueue(AsyncReceiver.QueueName);
                if (queueDescription == null)
                {
                    Console.WriteLine("\nUnexpected Error: QueueDescription is NULL");
                    return;
                }

                //*****************************************************************************************************
                //                                   Runtime Operations
                //*****************************************************************************************************
                factory = MessagingFactory.Create(serviceUri, credentials);

                QueueClient myQueueClient = factory.CreateQueueClient(AsyncReceiver.QueueName, ReceiveMode.PeekLock);

                //*****************************************************************************************************
                //                                   Receiving messages from a Queue
                //*****************************************************************************************************

                Console.WriteLine("\nReceiving messages from Queue '{0}'...", AsyncReceiver.QueueName);

                // Retreive the number of messages currently in the Queue
                long messageCount = queueDescription.MessageCount;

                // Initiate the Asynchronous Receive using BeginReceive() call to receive the messages. 
                for (long count = 0; count < messageCount; count++)
                {
                    myQueueClient.BeginReceive(TimeSpan.FromSeconds(30), OnMessageReceive, myQueueClient);
                }

                Console.WriteLine("\nAfter all messages are received, press ENTER to exit.");
                Console.ReadLine();
            }
            catch (Exception e)
            {
                Console.WriteLine("Unexpected exception {0}", e.ToString());
                throw;
            }
            finally
            {
                // Closing factory close all entities created from the factory.
                if (factory != null)
                {
                    factory.Close(); 
                }
            }
        }

        static void GetUserCredentials()
        {
            Console.Write("Please provide the service namespace to use: ");
            ServiceNamespace = Console.ReadLine();

            Console.Write("Please provide the issuer name to use: ");
            IssuerName = Console.ReadLine();

            Console.Write("Please provide the issuer key to use: ");
            IssuerKey = Console.ReadLine();
        }

        public static void OnMessageReceive(IAsyncResult result)
        {
            QueueClient queueClient = (QueueClient) result.AsyncState ;

            try
            {
                //Receive the message with the EndReceive call
                BrokeredMessage message = queueClient.EndReceive(result);

                if (message != null)
                {
                    Console.WriteLine("\nMessage Received: Id = {0}, Body = {1}", message.MessageId, message.GetBody<string>());

                    // The following is necessary in PeekLock Receive mode only.
                    // Complete the message asynchronously. 
                    message.BeginComplete(OnMessageComplete, message);
                }

                else
                {
                    Console.WriteLine("OnMessageReceive: Unexpected Error, Did not receive any messages");
                }
            }
            catch (Exception e)
            {
                Console.WriteLine("OnMessageReceive: Unexpected exception in {0}", e.ToString());
                throw;
            }
        }

        public static void OnMessageComplete(IAsyncResult result)
        {
            BrokeredMessage message = (BrokeredMessage) result.AsyncState ;

            // Signal the Completion of the Asynchronous Message Receive.
            try
            {
                message.EndComplete(result);
                Console.WriteLine("\nAsynchronous Message Receive Completed for Id = {0}", message.MessageId);
            }
            catch (Exception e)
            {
                Console.WriteLine("OnMessageComplete: Unexpected exception {0}", e.ToString());
                throw;
            }            
        }
    }
}
