//---------------------------------------------------------------------------------
// Copyright (c) 2011, Microsoft Corporation
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//---------------------------------------------------------------------------------

namespace Microsoft.Samples.AsyncMessaging
{
    using System;
    using System.Collections.Generic;
    using Microsoft.ServiceBus;
    using Microsoft.ServiceBus.Messaging;

    public class AsyncSender
    {
        const string QueueName = "IssueTrackingQueue";
        static string ServiceNamespace;
        static string IssuerName;
        static string IssuerKey;

        static void Main(string[] args)
        {
            GetUserCredentials();
            TokenProvider credentials = TokenProvider.CreateSharedSecretTokenProvider(AsyncSender.IssuerName, AsyncSender.IssuerKey);
            Uri serviceUri = ServiceBusEnvironment.CreateServiceUri("sb", AsyncSender.ServiceNamespace, string.Empty);

            MessagingFactory factory = null;

            try
            {
                //*****************************************************************************************************
                //                                   Management Operations
                //*****************************************************************************************************           
                NamespaceManager namespaceClient = new NamespaceManager(serviceUri, credentials);

                Console.WriteLine("\nCreating Queue '{0}'...", QueueName);

                // Delete if exists
                if (namespaceClient.QueueExists(AsyncSender.QueueName))
                {
                    namespaceClient.DeleteQueue(AsyncSender.QueueName);
                }

                namespaceClient.CreateQueue(QueueName);

                //*****************************************************************************************************
                //                                   Runtime Operations
                //*****************************************************************************************************
                factory = MessagingFactory.Create(serviceUri, credentials);

                QueueClient myQueueClient = factory.CreateQueueClient(AsyncSender.QueueName);

                //*****************************************************************************************************
                //                                   Sending messages to a Queue
                //*****************************************************************************************************
                List<BrokeredMessage> messageList = new List<BrokeredMessage>();

                messageList.Add(CreateIssueMessage("1", "First Package"));
                messageList.Add(CreateIssueMessage("2", "Second Package"));
                messageList.Add(CreateIssueMessage("3", "Third Package"));

                Console.WriteLine("\nSending messages to Queue...");

                foreach (BrokeredMessage message in messageList)
                {
                    // Initiate the asynchronous send 
                    myQueueClient.BeginSend(message, OnSendComplete, new Tuple<QueueClient, string>(myQueueClient, message.MessageId));
                    Console.WriteLine(string.Format("Asynchronous Message Send Begin: Id = {0}, Body = {1}", message.MessageId, message.GetBody<string>()));
                }

                Console.WriteLine("\nAfter all messages are sent, press ENTER to clean up and exit.\n");
                Console.ReadLine();
            }
            catch (Exception e)
            {
                Console.WriteLine("Unexpected exception {0}", e.ToString());
                throw;
            }
            finally
            {
                // Closing factory close all entities created from the factory.
                if(factory != null)
                    factory.Close();
            }            
        }

        static void GetUserCredentials()
        {
            Console.Write("Please provide the service namespace to use: ");
            ServiceNamespace = Console.ReadLine();

            Console.Write("Please provide the issuer name to use: ");
            IssuerName = Console.ReadLine();

            Console.Write("Please provide the issuer key to use: ");
            IssuerKey = Console.ReadLine();
        }

        private static BrokeredMessage CreateIssueMessage(string issueId, string issueBody)
        {
            BrokeredMessage message = new BrokeredMessage(issueBody);
            message.MessageId = issueId;
            return message;
        }

        public static void OnSendComplete(IAsyncResult result)
        {
            Tuple<QueueClient, string> stateInfo = (Tuple<QueueClient, string>) result.AsyncState ;

            QueueClient queueClient = stateInfo.Item1;
            string messageId = stateInfo.Item2;

            try
            {
                // Complete Asynchronous Message Send process
                queueClient.EndSend(result);
                Console.WriteLine("Asynchronous Message Send for Id = {0} Successful", messageId);
            }
            catch (Exception e)
            {
                Console.WriteLine("OnSendComplete: Asynchronous Message Send for Id = {0} Failed with Exception = {1}", messageId, e.ToString());
            }
        }
    }
}

