//---------------------------------------------------------------------------------
// Copyright (c) 2011, Microsoft Corporation
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//---------------------------------------------------------------------------------

namespace Microsoft.ServiceBus.Samples
{
    using System;
    using Microsoft.ServiceBus;
    using Microsoft.ServiceBus.Messaging;

    public class Receiver
    {
        private static string serviceNamespace, issuerName, issuerSecret;
        private const int MaxRetryCount = 5;

        private static System.Collections.Hashtable hashTable = new System.Collections.Hashtable();
        private enum OrderType
        {
            StayInOrder,
            TakeOutOrder,
            DeliveryOrder
        }

        public static void Main()
        {
            // Get credentials and create a client for receiving messages:
            GetUserCredentials();
            MessagingFactory messagingFactory = CreateMessagingFactory();
            QueueClient queueClient = messagingFactory.CreateQueueClient("OrdersService");

            // Read messages from queue until it is empty:
            Console.WriteLine("Reading messages from queue...");

            BrokeredMessage receivedMessage;
            while ((receivedMessage = queueClient.Receive(TimeSpan.FromSeconds(10))) != null)
            {
                int retryCount = 0;
                while (retryCount < MaxRetryCount)
                {
                    if (ProcessOrder(receivedMessage))
                        break;
                    else
                        retryCount++;
                }

                if (retryCount == MaxRetryCount)
                {
                    Console.WriteLine("Adding Order {0} with {1} number of items and {2} total to DeadLetter queue", receivedMessage.Properties["OrderNumber"],
                                receivedMessage.Properties["NumberOfItems"], receivedMessage.Properties["OrderTotal"]);
                    receivedMessage.DeadLetter("UnableToProcess", "Failed to process in reasonable attempts");
                }
            }

            Console.WriteLine();
            Console.WriteLine("No more messages left in queue. Logging dead lettered messages...");

            // Log the dead-lettered messages that could not be processed:
            QueueClient deadLetterClient = messagingFactory.CreateQueueClient(QueueClient.FormatDeadLetterPath(queueClient.Path), ReceiveMode.ReceiveAndDelete);
            BrokeredMessage receivedDeadLetterMessage;
            while ((receivedDeadLetterMessage = deadLetterClient.Receive(TimeSpan.FromSeconds(10))) != null)
            {
                LogOrder(receivedDeadLetterMessage);
            }
            
            Console.WriteLine();
            Console.WriteLine("Press [Enter] to exit.");
            Console.ReadLine();
        }

        private static void GetUserCredentials()
        {
            // User namespace
            Console.WriteLine("Please provide the namespace to use:");
            Receiver.serviceNamespace = Console.ReadLine().Trim();

            // Issuer name
            Console.WriteLine("Please provide the Issuer name to use:");
            Receiver.issuerName = Console.ReadLine().Trim();

            // Issuer key
            Console.WriteLine("Please provide the Issuer key to use:");
            Receiver.issuerSecret = Console.ReadLine().Trim();
        }

        private static MessagingFactory CreateMessagingFactory()
        {
            Uri runtimeAddress = ServiceBusEnvironment.CreateServiceUri("sb", Receiver.serviceNamespace, string.Empty);

            return MessagingFactory.Create(
                runtimeAddress,
                TokenProvider.CreateSharedSecretTokenProvider(Receiver.issuerName, Receiver.issuerSecret));
        }

        /// <summary>
        /// This method simulates the random failure behavior which happens in real world. 
        /// We will randomly select a message to fail based on some random number value. To make sure the message processing fails 
        /// all the times during subsequent retries, we add the result to the hashtable and retrieve it from there. 
        /// </summary>
        /// <param name="receivedMessage"></param>
        /// <returns></returns>
        private static bool ProcessOrder(BrokeredMessage receivedMessage)
        {
            if (hashTable.ContainsKey(receivedMessage.Properties["OrderNumber"]))
            {
                return false;
            }

            if (new Random().Next() % 2 == 0 ? true : false)
            {
                Console.WriteLine("Received Order {0} with {1} number of items and {2} total", receivedMessage.Properties["OrderNumber"],
                                 receivedMessage.Properties["NumberOfItems"], receivedMessage.Properties["OrderTotal"]);
                return true;
            }
            else
            {
                hashTable.Add(receivedMessage.Properties["OrderNumber"], false);
                return false;
            }
        }

        private static void LogOrder(BrokeredMessage message)
        {
            Console.WriteLine("Order {0} with {1} number of items and {2} total logged from DeadLetter queue. DeadLettering Reason is \"{3}\" and Deadlettering error description is \"{4}\"", message.Properties["OrderNumber"],
                                  message.Properties["NumberOfItems"], message.Properties["OrderTotal"], message.Properties["DeadLetterReason"], message.Properties["DeadLetterErrorDescription"]);
        }
    }
}
