//---------------------------------------------------------------------------------
// Copyright (c) 2011, Microsoft Corporation
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//---------------------------------------------------------------------------------

namespace Microsoft.ServiceBus.Samples
{
    using System;
    using System.Collections.Generic;
    using Microsoft.ServiceBus;
    using Microsoft.ServiceBus.Description;
    using Microsoft.ServiceBus.Messaging;
    
    public class Sender
    {
        private static NamespaceManager namespaceManager;
        private static MessagingFactory messagingFactory;

        private static string serviceBusNamespace;
        private static string serviceBusIssuerName;
        private static string serviceBusIssuerKey;

        public static void Main()
        {
            // Setup:
            Sender.GetUserCredentials();
            QueueDescription queueDescription = CreateQueue();
            QueueClient queueClient = CreateQueueClient(queueDescription);

            // Send messages to queue:
            Console.WriteLine("Sending messages to queue...");

            BrokeredMessage message1 = CreateOrderMessage("High");
            queueClient.Send(message1);
            Console.WriteLine("Sent message {0} with high priority.", message1.MessageId);

            BrokeredMessage message2 = CreateOrderMessage("Low");
            queueClient.Send(message2);
            Console.WriteLine("Sent message {0} with low priority.", message2.MessageId);

            BrokeredMessage message3 = CreateOrderMessage("High");
            queueClient.Send(message3);
            Console.WriteLine("Sent message {0} with high priority.", message3.MessageId);

            Console.WriteLine();
            Console.WriteLine("Press [Enter] to delete queue and exit.");
            Console.ReadLine();

            // Cleanup:
            messagingFactory.Close();
            namespaceManager.DeleteQueue(queueDescription.Path);
        }

        private static void GetUserCredentials()
        {
            // User namespace
            Console.WriteLine("Please provide the namespace to use:");
            Sender.serviceBusNamespace = Console.ReadLine().Trim();

            // Issuer name
            Console.WriteLine("Please provide the Issuer name to use:");
            Sender.serviceBusIssuerName = Console.ReadLine().Trim();

            // Issuer key
            Console.WriteLine("Please provide the Issuer key to use:");
            Sender.serviceBusIssuerKey = Console.ReadLine().Trim();
        }

        private static QueueDescription CreateQueue()
        {
            Uri managementAddress = ServiceBusEnvironment.CreateServiceUri("https", Sender.serviceBusNamespace, string.Empty);
            namespaceManager = new NamespaceManager(
                managementAddress,
                TokenProvider.CreateSharedSecretTokenProvider(Sender.serviceBusIssuerName, Sender.serviceBusIssuerKey));

            Console.WriteLine("Creating queue \"OrdersQueue\".");

            if (namespaceManager.QueueExists("OrdersQueue"))
            {
                namespaceManager.DeleteQueue("OrdersQueue");
            }

            return namespaceManager.CreateQueue("OrdersQueue");
        }

        private static QueueClient CreateQueueClient(QueueDescription queueDescription)
        {
            Uri runtimeUri = ServiceBusEnvironment.CreateServiceUri("sb", Sender.serviceBusNamespace, string.Empty);
            messagingFactory = MessagingFactory.Create(
                runtimeUri,
                TokenProvider.CreateSharedSecretTokenProvider(Sender.serviceBusIssuerName, Sender.serviceBusIssuerKey));

            return messagingFactory.CreateQueueClient(queueDescription.Path);
        }

        private static BrokeredMessage CreateOrderMessage(string priority)
        {
            BrokeredMessage message = new BrokeredMessage();
            message.MessageId = "Order" + Guid.NewGuid().ToString();
            message.Properties.Add("Priority", priority);
            return message;
        }
    }
}
