//---------------------------------------------------------------------------------
// Copyright (c) 2011, Microsoft Corporation
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//---------------------------------------------------------------------------------
using System;
using System.Diagnostics;
using Microsoft.ServiceBus;
using Microsoft.ServiceBus.Messaging;

namespace Microsoft.Samples.MessagesPrefetchSample
{
    class Program
    {
        #region Fields
        static string serviceBusNamespace;
        static string serviceBusIssuerName;
        static string serviceBusIssuerKey;

        const string QueueName = "MyQueue";
        #endregion

        static void Main(string[] args)
        {
            // ***************************************************************************************
            // This sample demonstrates how to use the messages prefetch feature upon receive
            // The sample creates a Queue, sends messages to it and receives all messages
            // using 2 receivers one with prefetchCount = 0 (disabled) and the other with 
            // prefetCount = 100. For each case, it calculates the time taken to receive and complete
            // all messages and at the end, it prints the difference between both times.
            // ***************************************************************************************

            // Get ServiceBus namespace and credentials from the user.
            Program.GetNamespaceAndCredentials();

            // Create mesasging factory and ServiceBus namespace manager.
            MessagingFactory messagingFactory = Program.CreateMessagingFactory();
            NamespaceManager namespaceManager = Program.CreateNamespaceManager();

            // Create queue that will be used through the sample.
            Program.CreateQueue(namespaceManager);

            // Send and Receive messages with prefetch OFF
            long timeTaken1 = Program.SendAndReceiveMessages(messagingFactory, 0);

            // Send and Receive messages with prefetch ON
            long timeTaken2 = Program.SendAndReceiveMessages(messagingFactory, 100);

            // Calculate the time difference
            long timeDifference = timeTaken1 - timeTaken2;

            Console.WriteLine("\nTime difference = {0} milliseconds", timeDifference);

            Console.WriteLine();
            Console.WriteLine("Press [Enter] to quit...");
            Console.ReadLine();

            // Cleanup:
            messagingFactory.Close();
            namespaceManager.DeleteQueue(Program.QueueName);
        }

        #region Helper Methods

        static long SendAndReceiveMessages(MessagingFactory messagingFactory, int prefetchCount)
        {
            // Create client for the queue.
            QueueClient queueClient = messagingFactory.CreateQueueClient(Program.QueueName, ReceiveMode.PeekLock);

            // Now we can start sending messages.
            int messageCount = 1000;

            Console.WriteLine("\nSending {0} messages to the queue", messageCount);

            for (int i = 0; i < messageCount; i++)
            {
                queueClient.Send(new BrokeredMessage());
            }

            Console.WriteLine("Send completed");

            // Set the prefetchCount on the queueClient
            queueClient.PrefetchCount = prefetchCount;

            // Start stopwatch
            Stopwatch stopWatch = new Stopwatch();
            stopWatch.Start();

            // Receive the messages
            Console.WriteLine("Receiving messages from queue using prefetchCount = {0}", prefetchCount);

            BrokeredMessage receivedMessage = queueClient.Receive(TimeSpan.FromSeconds(10));

            while (receivedMessage != null)
            {
                receivedMessage.Complete();
                receivedMessage = queueClient.Receive(TimeSpan.FromSeconds(10));
            }

            Console.WriteLine("Receive completed");

            // Stop the stopwatch
            stopWatch.Stop();

            long timeTaken = stopWatch.ElapsedMilliseconds;
            Console.WriteLine("Time to receive and complete all messages = {0} milliseconds", timeTaken);

            // Close the QueueClient
            queueClient.Close();

            return timeTaken;
        }

        static void CreateQueue(NamespaceManager namespaceManager)
        {
            Console.WriteLine("\nCreating a queue.");
            
            // Create a queue.
            if(namespaceManager.QueueExists(Program.QueueName))
            {
                namespaceManager.DeleteQueue(Program.QueueName);
            }

            QueueDescription description = new QueueDescription(Program.QueueName);
            description.LockDuration = TimeSpan.FromMinutes(3);

            QueueDescription queueDescription = namespaceManager.CreateQueue(description);

            Console.WriteLine("Queue created.");
        }

        static NamespaceManager CreateNamespaceManager()
        {
            Uri managementAddress =  ServiceBusEnvironment.CreateServiceUri("https", Program.serviceBusNamespace, string.Empty);

            return new NamespaceManager(
                managementAddress,
                TokenProvider.CreateSharedSecretTokenProvider(Program.serviceBusIssuerName, Program.serviceBusIssuerKey));
        }

        static MessagingFactory CreateMessagingFactory()
        {
            Uri runtimeAddress = ServiceBusEnvironment.CreateServiceUri("sb", Program.serviceBusNamespace, string.Empty);

            return MessagingFactory.Create(
                runtimeAddress,
                TokenProvider.CreateSharedSecretTokenProvider(Program.serviceBusIssuerName, Program.serviceBusIssuerKey));
        }

        static void GetNamespaceAndCredentials()
        {
            // User namespace
            Console.WriteLine("Please provide the namespace to use:");
            Program.serviceBusNamespace = Console.ReadLine().Trim();
            
            // Issuer name
            Console.WriteLine("Please provide the Issuer name to use:");
            Program.serviceBusIssuerName = Console.ReadLine().Trim();

            // Issuer key
            Console.WriteLine("Please provide the Issuer key to use:");
            Program.serviceBusIssuerKey = Console.ReadLine().Trim();
        }
        #endregion
    }
}
