//---------------------------------------------------------------------------------
// Copyright (c) 2011, Microsoft Corporation
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//---------------------------------------------------------------------------------

namespace Microsoft.ServiceBus.Samples.MsmqServiceBusBridge
{
    using System;
    using System.Messaging;
    using Microsoft.ServiceBus;
    using Microsoft.ServiceBus.Messaging;

    public class Helper
    {
        private static string serviceNamespace, issuerName, issuerSecret;

        public static void GetUserCredentials()
        {
            //Read user credentials.
            Console.Write("Service Namespace: ");
            serviceNamespace = Console.ReadLine();

            Console.Write("Issuer Name: ");
            issuerName = Console.ReadLine();

            Console.Write("Issuer Secret: ");
            issuerSecret = Console.ReadLine();
            Console.WriteLine();
        }

        public static Uri GetNamespace()
        {
            if (string.IsNullOrEmpty(serviceNamespace))
            {
                GetUserCredentials();
            }

            Uri namespaceUri = ServiceBusEnvironment.CreateServiceUri("sb", serviceNamespace, string.Empty);
            return namespaceUri;
        }

        public static TokenProvider GetTokenProvider()
        {
            if (string.IsNullOrEmpty(issuerName) || string.IsNullOrEmpty(issuerSecret))
            {
                GetUserCredentials();
            }

            return TokenProvider.CreateSharedSecretTokenProvider(issuerName, issuerSecret);
        }

        public static QueueDescription CreateServiceBusQueue(string queueName)
        {
            QueueDescription queueDesc; 
            NamespaceManager namespaceClient = GetNamespaceManager();
            if (namespaceClient.QueueExists(queueName))
            {
                queueDesc = namespaceClient.GetQueue(queueName);
            }
            else
            {
                queueDesc = namespaceClient.CreateQueue(queueName);
            }

            return queueDesc;
        }

        public static void DeleteQueue(string queueName)
        {
            NamespaceManager namespaceClient = GetNamespaceManager();
            if (namespaceClient.QueueExists(queueName))
            {
                namespaceClient.DeleteQueue(queueName);
            }
        }

        public static NamespaceManager GetNamespaceManager()
        {
            Uri nameSpaceUri = GetNamespace();
            TokenProvider clientToken = GetTokenProvider();
            return new NamespaceManager(nameSpaceUri, clientToken);
        }

        public static MessagingFactory GetMessagingFactory()
        {
            // Create a MessagingFactory to receive messages.
            Uri nameSpaceUri = GetNamespace();
            TokenProvider clientToken = GetTokenProvider();
            return MessagingFactory.Create(nameSpaceUri, clientToken);
        }

        public static QueueClient GetServiceBusQueueClient(MessagingFactory messagingFactory, string queueName, ReceiveMode receiveMode = ReceiveMode.ReceiveAndDelete)
        {
            QueueClient client =  messagingFactory.CreateQueueClient(queueName, receiveMode);
            return client;
        }

        public static void CreateMsmqQueue(string queueName)
        {
            if (!MessageQueue.Exists(queueName))
            {
                MessageQueue.Create(queueName, true);
            }
        }

        public static void DeleteMsmqQueue(string queueName)
        {
            if (MessageQueue.Exists(queueName))
            {
                MessageQueue.Delete(queueName);
            }
        }

        public static MessageQueue OpenMsmqQueue(string queueName, bool sharedModeDenyReceive = false)
        {
            var msmqQueue = new System.Messaging.MessageQueue(queueName, sharedModeDenyReceive);
            msmqQueue.MessageReadPropertyFilter.SetAll();
            msmqQueue.Refresh();
            msmqQueue.Formatter = new BinaryMessageFormatter();
            return msmqQueue;
        }
    }
}
