//---------------------------------------------------------------------------------
// Copyright (c) 2011, Microsoft Corporation
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//---------------------------------------------------------------------------------

namespace Microsoft.ServiceBus.Samples.RequestResponse
{
    using System;

    using Microsoft.ServiceBus;
    using Microsoft.ServiceBus.Messaging;

    class Program
    {
        #region Fields
        static string ServiceBusNamespace;
        static string ServiceBusIssuerName;
        static string ServiceBusIssuerKey;
        static string ClientId;

        const double ResponseMessageTimeout = 20.0;
        #endregion

        static void Main(string[] args)
        {
            ParseArgs(args);
            Console.Title = "Client";

            // Send request messages to request queue
            QueueClient requestClient = CreateQueueClient(SampleManager.RequestQueuePath);
            QueueClient responseClient = CreateQueueClient(SampleManager.ResponseQueuePath);
            MessageSession session = responseClient.AcceptMessageSession(ClientId);
            Console.WriteLine("Preparing to send request messages to {0}...", requestClient.Path);

            SendMessages(requestClient, responseClient, session);

            // All messages sent
            Console.WriteLine("\nClient finished sending requests.");
            Console.ReadLine();
        }

        // Create the runtime entities (queue client)
        static QueueClient CreateQueueClient(string queuePath)
        {
            TokenProvider tokenProvider = TokenProvider.CreateSharedSecretTokenProvider(
                ServiceBusIssuerName, ServiceBusIssuerKey);

            Uri uri = ServiceBusEnvironment.CreateServiceUri("sb", ServiceBusNamespace, string.Empty);
            MessagingFactory messagingFactory = MessagingFactory.Create(uri, tokenProvider);

            return messagingFactory.CreateQueueClient(queuePath, ReceiveMode.ReceiveAndDelete);
        }

        static void SendMessages(QueueClient requestClient, QueueClient responseClient, MessageSession session)
        {
            // Send request messages to queue:
            Console.WriteLine("Sending request messages to queue {0}", requestClient.Path);
            Console.WriteLine("Receiving response messages on queue {0}", responseClient.Path);

            for (int i = 0; i < SampleManager.NumMessages; ++i)
            {
                // send request message
                BrokeredMessage message = new BrokeredMessage { ReplyToSessionId = ClientId, MessageId = i.ToString() };
                
                requestClient.Send(message);
                SampleManager.OutputMessageInfo("REQUEST: ", message);

                // start asynchronous receive operation
                session.BeginReceive(TimeSpan.FromSeconds(ResponseMessageTimeout), ProcessResponse, session);
            }


            Console.WriteLine();
        }

        static void ParseArgs(string[] args)
        {
            if (args.Length != 4)
            {
                throw new ArgumentException("Incorrect number of arguments. args = {0}", args.ToString());
            }

            ServiceBusNamespace = args[0];
            ServiceBusIssuerName = args[1];
            ServiceBusIssuerKey = args[2];
            ClientId = args[3];
        }

        static void ProcessResponse(IAsyncResult result)
        {
            MessageSession session = result.AsyncState as MessageSession;
            BrokeredMessage message = session.EndReceive(result);

            if (message == null)
            {
                Console.WriteLine("ERROR: Message Receive Timeout.");
            }
            else
            {
                SampleManager.OutputMessageInfo("RESPONSE: ", message);
            }
        }
    }
}
