//---------------------------------------------------------------------------------
// Copyright (c) 2011, Microsoft Corporation
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//---------------------------------------------------------------------------------

namespace Microsoft.ServiceBus.Samples.RequestResponse
{
    using System;

    using Microsoft.ServiceBus;
    using Microsoft.ServiceBus.Messaging;

    class Program
    {
        #region Fields
        static string ServiceBusNamespace;
        static string ServiceBusIssuerName;
        static string ServiceBusIssuerKey;

        const double ReceiveMessageTimeout = 20.0;
        #endregion

        static void Main(string[] args)
        {
            ParseArgs(args);

            // Receive request messages from request queue
            Console.Title = "Server";
            TopicClient topicClient = CreateTopicClient(SampleManager.TopicPath);
            SubscriptionClient requestClient = CreateSubscriptionClient(
                SampleManager.TopicPath, SampleManager.RequestSubName);

            Console.WriteLine("Ready to receive messages from {0}/{1}...", requestClient.TopicPath, requestClient.Name);
            ReceiveMessages(topicClient, requestClient);

            Console.WriteLine("\nServer complete.");
            Console.ReadLine();
        }

        static void ReceiveMessages(TopicClient topicClient, SubscriptionClient requestClient)
        {
            // Read all the messages from subscription:
            BrokeredMessage request;
            while ((request = requestClient.Receive(TimeSpan.FromSeconds(ReceiveMessageTimeout))) != null)
            {
                SampleManager.OutputMessageInfo("REQUEST: ", request);

                BrokeredMessage response = new BrokeredMessage
                    {
                        SessionId = request.ReplyToSessionId,
                        MessageId = request.MessageId,
                        CorrelationId = SampleManager.ResponseSubName
                    };

                topicClient.Send(response);
                SampleManager.OutputMessageInfo("RESPONSE: ", response);
            }
        }

        public static TopicClient CreateTopicClient(string topicPath)
        {
            TokenProvider tokenProvider = TokenProvider.CreateSharedSecretTokenProvider(
                ServiceBusIssuerName, ServiceBusIssuerKey);

            Uri uri = ServiceBusEnvironment.CreateServiceUri("sb", ServiceBusNamespace, string.Empty);
            MessagingFactory messagingFactory = MessagingFactory.Create(uri, tokenProvider);

            return messagingFactory.CreateTopicClient(topicPath);
        }

        public static SubscriptionClient CreateSubscriptionClient(string topicPath, string subName)
        {
            TokenProvider tokenProvider = TokenProvider.CreateSharedSecretTokenProvider(
                ServiceBusIssuerName, ServiceBusIssuerKey);

            Uri uri = ServiceBusEnvironment.CreateServiceUri("sb", ServiceBusNamespace, string.Empty);
            MessagingFactory messagingFactory = MessagingFactory.Create(uri, tokenProvider);

            return messagingFactory.CreateSubscriptionClient(topicPath, subName, ReceiveMode.ReceiveAndDelete);
        }

        static void ParseArgs(string[] args)
        {
            if (args.Length != 3)
            {
                throw new ArgumentException("Incorrect number of arguments. args = {0}", args.ToString());
            }

            ServiceBusNamespace = args[0];
            ServiceBusIssuerName = args[1];
            ServiceBusIssuerKey = args[2];
        }
    }
}
