//---------------------------------------------------------------------------------
// Copyright (c) 2011, Microsoft Corporation
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//---------------------------------------------------------------------------------

namespace Microsoft.Samples.ServiceBus.WindowsPhoneChat.ViewModels
{
    using System;
    using System.Windows;
    using Microsoft.Samples.ServiceBus.Messaging;
    using Microsoft.Samples.ServiceBus.WindowsPhoneChat.Commands;
    using Microsoft.Samples.ServiceBus.WindowsPhoneChat.Controllers;

    public class ChatTopicViewModel : ViewModel
    {
        public static readonly DependencyProperty IsSubscribedProperty =
            DependencyProperty.Register("IsSubscribed", typeof(bool), typeof(ChatTopicViewModel), new PropertyMetadata(false));

        private readonly TopicClient topicClient;
        private readonly ChatTopicListControlViewModel owner;

        private SubscriptionClient subscriptionClient;

        public ChatTopicViewModel(TopicClient topicClient, ChatTopicListControlViewModel owner)
        {
            if (topicClient == null)
            {
                throw new ArgumentNullException("topicClient");
            }

            if (owner == null)
            {
                throw new ArgumentNullException("owner");
            }

            this.topicClient = topicClient;
            this.owner = owner;
        }

        public event EventHandler SubscriptionChanged;

        public string Name
        {
            get { return this.topicClient.Path; }
        }

        public TopicClient TopicClient
        {
            get { return this.topicClient; }
        }

        public ChatTopicListControlViewModel Owner
        {
            get { return this.owner; }
        }

        public bool IsSubscribed
        {
            get { return (bool)GetValue(IsSubscribedProperty); }
            set { SetValue(IsSubscribedProperty, value); }
        }

        public SubscriptionClient SubscriptionClient
        {
            get
            {
                return this.subscriptionClient;
            }

            set
            {
                if (this.subscriptionClient != value)
                {
                    if (value != null)
                    {
                        value.ReceiveMode = ReceiveMode.PeekLock;
                    }

                    this.subscriptionClient = value;
                    this.IsSubscribed = value != null;
                    this.OnSubscriptionChanged();
                }
            }
        }

        internal void OnSelected()
        {
            var viewModel = new ChatPageViewModel(this);
            ViewController.Current.NavigateToPage(viewModel);
        }

        internal void BeginSubscribe()
        {
            if (Commands.SubscribeToTopic.CanExecute(this))
            {
                Commands.SubscribeToTopic.Execute(this);
            }
        }

        internal void BeginDelete()
        {
            if (Commands.DeleteTopic.CanExecute(this))
            {
                Commands.DeleteTopic.Execute(this);
            }
        }

        private void OnSubscriptionChanged()
        {
            if (this.SubscriptionChanged != null)
            {
                this.SubscriptionChanged(this, EventArgs.Empty);
            }
        }
    }
}
